/*
*	Audio Alert v2.3, Copyright Jonathan Mackey 2018
*
*
*	GNU license:
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*	Please maintain this license information along with authorship and copyright
*	notices in any redistribution of this code.
*
*	This code uses a slightly modified version of Felix Rusu's RFM69 library.
*	Copyright Felix Rusu 2016, http://www.LowPowerLab.com/contact
*
*	This code uses Andy Karpov's Arduino PT2314 Library
*	Copyright Andy Karpov 2013 <andy.karpov@gmail.com>
*/

#include <RFM69.h>
#include <SPI.h>
#include <Wire.h>
#include <DFPlayer.h>
#include <PT2314.h>
#include <SoftwareSerial.h>

/*******************************************************************************
***** IMPORTANT SETTINGS - YOU MUST CHANGE/CONFIGURE TO FIT YOUR HARDWARE ******
*******************************************************************************/
#define NODEID        1    //unique for each node on same network
#define NETWORKID     100  //the same on all nodes that talk to each other
//Match frequency to the hardware version of your RFM69
#define FREQUENCY     RF69_433MHZ
//#define FREQUENCY     RF69_868MHZ
//#define FREQUENCY     RF69_915MHZ
//#define DCF_RECEIVED_PIN 4

RFM69 radio;
#define RxPin 5
#define TxPin 6
SoftwareSerial swSerial(RxPin, TxPin); // RX, TX

DFPlayer dfPlayer(swSerial);
PT2314	audioMux;
int currentChannel = -1;
/*********************************** setup ************************************/
void setup(void)
{
#ifdef DCF_RECEIVED_PIN
	pinMode(DCF_RECEIVED_PIN, OUTPUT);
	digitalWrite(DCF_RECEIVED_PIN, LOW);
#endif
	delay(1000);
	Wire.begin();
	delay(1000);
	Serial.begin(9600);		// Must be the same as swSerial
	swSerial.begin(9600);	// swSerial must be 9600 for the DFPlayer
	dfPlayer.begin();
	delay(10);
	radio.initialize(FREQUENCY,NODEID,NETWORKID);
	Serial.print("Starting..., audioMux initialized = ");
	Serial.println(audioMux.init() ? "YES":"NO");
	
	audioMux.channel(0);   // switch to channel 1... 0-3 => channel 1-4
	audioMux.volume(90);
	delay(3000); // Delay 3 seconds to allow the DF Player to read the device.
}

uint32_t packetCount = 0;

uint32_t StrToUInt32(
	const char* inStr);
uint16_t ReadDecFromSerial(void);
uint8_t ReadHexFromSerial(void);

			
/******************************* PlayAlert ************************************/
void PlayAlert(
	uint16_t	inFileIndex)
{
	currentChannel = audioMux.GetCurrentChannel();
	audioMux.volume(75);
	audioMux.channel(1);
	dfPlayer.PlayNthRootFile(inFileIndex);
}

/*************************** DumpDFPlayerResponse *****************************/
void DumpDFPlayerResponse(void)
{
	Serial.print("Response Command = 0x");
	Serial.print(dfPlayer.GetCommand(), HEX);
	dfPlayer.ClearReplyCommand();
	Serial.print(", Param = 0x");
	Serial.println(dfPlayer.GetParam(), HEX);
}

/*********************************** loop *************************************/
void loop(void)
{
	if (radio.receiveDone())
	{
		Serial.print("#[");
		Serial.print(++packetCount);
		Serial.print(']');
		Serial.print('[');Serial.print(radio.SENDERID, DEC);
		Serial.print("] ");
		for (byte i = 0; i < radio.DATALEN; i++)
			Serial.print((char)radio.DATA[i]);
		Serial.print("	 [RX_RSSI:");
		Serial.print(radio.RSSI);
		Serial.println("]");

		uint32_t	dataValue = StrToUInt32(radio.DATA);
#ifdef DCF_RECEIVED_PIN
		digitalWrite(DCF_RECEIVED_PIN, dataValue == 0xDCF ? HIGH:LOW);
#endif
		if (radio.ACKRequested())
		{
			radio.sendACK();
		}
		switch (dataValue)
		{
			case  0xDCF:	// Dust Collector Full
				PlayAlert(1);
				break;
			case 0xAFD:		// Someone is at the front door
				PlayAlert(2);
				break;
			case 0xABD:
				PlayAlert(3); // Someone is at the back door
				break;
			default:
				break;
		}
	} else if (currentChannel >= 0 &&
		dfPlayer.CommandCompleted())
	{
		audioMux.channel(currentChannel);
		currentChannel = -1;
		audioMux.volume(90);
		DumpDFPlayerResponse();
	} else if (dfPlayer.CommandCompleted())
	{
		DumpDFPlayerResponse();
	}
	if (Serial.available() > 0)
	{
		delay(500);	// Allow time for the rest of the string to arrive
		while(Serial.available())
		{
			switch(Serial.read())
			{
				case 'v':
					audioMux.volume(ReadDecFromSerial());
					break;
				case 'c':
					audioMux.channel(ReadDecFromSerial());
					break;
				case 'M':
					audioMux.muteOn();
					break;
				case 'm':
					audioMux.muteOff();
					break;
				case 'L':
					audioMux.loudnessOn();
					break;
				case 'l':
					audioMux.loudnessOff();
					break;
				case 'g':
					audioMux.gain(ReadDecFromSerial());
					break;
				case 'a':
				{
					int attn = ReadDecFromSerial();
					audioMux.attenuation(attn, attn);
					break;
				}
				case 'b':
					audioMux.bass(ReadDecFromSerial());
					break;
				case 't':
					audioMux.treble(ReadDecFromSerial());
					break;
				case 'p':
					PlayAlert(ReadDecFromSerial());
					break;
				case '>':
				{
					uint8_t	command = ReadHexFromSerial();
					if (Serial.read() == ' ')
					{
						uint16_t	param = ReadHexFromSerial()<<8;
						if (Serial.read() == ' ')
						{
							param += ReadHexFromSerial();
							Serial.print("Sending Command = 0x");
							Serial.print(command, HEX);
							Serial.print(", Param = 0x");
							Serial.println(param, HEX);
							dfPlayer.SendCommand(command, param, true);
						}
					}
					break;
				}
				default:
					break;
			}
		}
	}
}

/**************************** ReadDecFromSerial *******************************/
uint16_t ReadDecFromSerial(void)
{
	uint16_t	decVal = 0;
	for (uint8_t i = 4; i > 0 && Serial.available(); i--)
	{
		int thisByte = Serial.peek();
		if (thisByte >= '0' && thisByte <= '9')
		{
			Serial.read();
			decVal = (decVal * 10) + (thisByte - '0');
		} else
		{
			break;
		}
	}
	return(decVal);
}

/**************************** ReadHexFromSerial *******************************/
uint8_t ReadHexFromSerial(void)
{
	uint8_t	hexVal = 0;
	for (uint8_t i = 2; i > 0 && Serial.available(); i--)
	{
		int thisByte = Serial.peek();
		if (thisByte >= '0' && thisByte <= '9')
		{
			Serial.read();
			hexVal <<= 4;
			hexVal += (thisByte - '0');
		} else if ((thisByte >= 'a' && thisByte <= 'f') ||
			(thisByte >= 'A' && thisByte <= 'F'))
		{
			Serial.read();
			hexVal <<= 4;
			hexVal += ((thisByte | 0x20) - 'a' + 10);
		} else
		{
			break;
		}
	}
	return(hexVal);
}

/******************************* StrToUInt32 **********************************/
uint32_t StrToUInt32(
	const char* inStr)
{
	uint32_t	value = 0;
	uint8_t		thisByte = (uint8_t)*(inStr++);
	for (uint8_t i = 0; i < 8 && thisByte; i++)
	{
		if (thisByte >= '0' && thisByte <= '9')
		{
			value <<= 4;
			value += (thisByte - '0');
		} else if ((thisByte >= 'A' && thisByte <= 'F'))
		{
			value <<= 4;
			value += ((thisByte | 0x20) - 'a' + 10);
		} else
		{
			break;
		}
		thisByte = (uint8_t)*(inStr++);
	}
	return(value);
}
